package at.ac.uibk.gitsearch.web.rest;

import static org.assertj.core.api.Assertions.fail;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.service.dto.OAuth2ConfigDTO;
import nl.jqno.equalsverifier.EqualsVerifier;
import nl.jqno.equalsverifier.Warning;

/**
 * Integration tests for the {@link UserResource} REST controller.
 */
@AutoConfigureMockMvc
@SpringBootTest(classes = GitsearchApp.class)
public class OAuth2ConfigResourceIT {

    @Autowired
    private MockMvc restMockMvc;



    @Test
    public void testGetOAuth2Configs() throws Exception {
    	restMockMvc.perform(get("/oauth2Config/allConfigs")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
 	   .andExpect(content().contentType(MediaType.APPLICATION_JSON))
       .andExpect(status().isOk())
		.andExpect(TestUtil.testResult(OAuth2ConfigDTO[].class, r -> {
			assertTrue("At least one entry", r.length > 0);
			for(OAuth2ConfigDTO config: r)
				try {
					testGetOAuth2Config(config);
				} catch (Exception e) {
					fail("Exception " + e.getMessage() + " not expected!");
				}
			
			
			
		}));
    }

    public void testGetOAuth2Config(OAuth2ConfigDTO config) throws Exception {
    	restMockMvc.perform(get("/oauth2Config//config/"+ config.getRegistrationId())
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
 	   .andExpect(content().contentType(MediaType.APPLICATION_JSON))
       .andExpect(status().isOk())
		.andExpect(TestUtil.testResult(OAuth2ConfigDTO.class, r -> {
			assertEquals("Should be the same configs", config, r);
		}));
    }
    
    @Test
    public void testEqualsOnOAuth2ConfigDTO() {
		EqualsVerifier.forClass(OAuth2ConfigDTO.class).suppress(Warning.NONFINAL_FIELDS).verify();
    }



}
